</tbody>
</table>
<input type="hidden" name="actionID" value="" />
<input type="hidden" name="show" value="<?php echo htmlspecialchars(Horde_Util::getFormData('show', 'all')) ?>" />
<input type="hidden" name="key" value="<?php echo htmlspecialchars(Horde_Util::getFormData('key')) ?>" />
<input type="hidden" name="targetAddressbook" value="" />
<input type="hidden" name="targetList" value="" />
<input type="hidden" name="targetNew" value="" />
</form>
