.QuickFinderNoMatch {
    display: none;
}
#QuickFinderEmpty {
    font-style: italic;
}

#quicksearch {
    display: inline;
}
#quicksearch input {
    background-image: url("graphics/search.png");
    background-repeat: no-repeat;
    background-position: 2px center;
    padding: 1px 0 1px 20px;
}
#quicksearch a {
    display: inline-block;
    padding: 2px 4px;
}

.turbaPager {
    text-align: center;
    padding: 3px;
    background: #fff;
    border-bottom: 1px solid #ccc;
}

#vbook-form {
    margin: 1em 0 0 0;
}

div.editGroupMembers {
    font-weight: bold;
    padding: 8px 2px;
}
div.editGroupMembers span.iconImg {
    margin-right: 4px;
}

/* Images */
.editImg {
    background-image: url("graphics/edit.png");
}
.groupImg {
    background-image: url("graphics/group.png");
}
.searchImg {
    background-image: url("graphics/search.png");
}
.vcardImg {
    background-image: url("graphics/mime/vcard.png");
}
#tabContact a {
    background-image: url("graphics/turba.png");
}
.turba-browse {
    background-image: url("graphics/menu/browse.png");
}
.turba-delete, #tabDeleteContact a {
    background-image: url("graphics/delete.png");
}
.turba-edit, #tabEditContact a {
    background-image: url("graphics/edit.png");
}
.turba-search {
    background-image: url("graphics/search.png");
}

/* Tables. */
table#addressbook-list {
    width: 99%;
    margin: 0 0 8px 5px;
    border-top: 1px solid #ddd;
    border-left: 1px solid #ddd;
}
table#addressbook-list th {
    padding: 3px;
    background: #e9e9e9;
    border-right: 1px solid #ccc;
    text-align: left;
}
table#addressbook-list td {
    padding: 3px;
    border-right: 1px solid #ddd;
    border-bottom: 1px solid #ddd;
}
table#addressbook-list th.sortup {
    background: #bbcbff url("graphics/za.png") center left no-repeat;
    padding-left: 10px;
}
table#addressbook-list th.sortdown {
    background: #bbcbff url("graphics/az.png") center left no-repeat;
    padding-left: 10px;
}

#addressbook-list-buttons {
    padding: 1em;
}
#addressbook-list-buttons form {
    display: inline;
}
.addressbook-list-icon {
    width: 1%;
}

.turba-browse-icon {
    text-align: center;
}

/* Duplicates */
.turba-duplicate {
    overflow: auto;
    white-space: nowrap;
    vertical-align: top;
    max-height: 400px;
}
.turba-duplicate-contact {
    display: inline-block;
    margin-top: 8px;
    margin-right: 8px;
}
.turba-duplicate-contact p {
    margin: 8px;
}
.turba-duplicate-contact form {
    display: inline;
}
.turba-duplicate-forms {
    margin: 8px;
}

/* Search page */
#advancedtoggle {
    background-color: #ffa;
    margin: -12px;
    padding: 12px;
}

/* Preferences pages */
#turba-prefs-cols-list {
    float: left;
    width: 15em;
}
#turba-prefs-cols-list ul {
    list-style-type: none;
}
#turba-prefs-cols-list li {
    padding: 3px;
}
#turba-prefs-cols-list li a {
    display: block;
}
#turba-prefs-cols-list li.active {
    font-weight: bold;
    border: 1px solid #ccc;
    border-right: none;
}

#turba-prefs-cols-columns {
    float: left;
    padding: 3px;
    border: 1px solid #ccc;
}
#turba-prefs-cols-columns ol {
    list-style-type: none;
}
#turba-prefs-cols-columns li {
    padding: 2px;
    cursor: move;
}
#turba-prefs-cols-columns .checkbox {
    margin-right: 2px;
    cursor: default;
}

@media print {
    .tabset, #page h1.header span.smallheader {
        display: none !important;
    }
}
